# -*- coding: utf-8 -*-
#
# This file is part of COUNTER-Robots.
# Copyright (C) 2018 CERN.
#
# COUNTER-Robots is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Small script to update the list of robots and machines."""

from __future__ import absolute_import, print_function

import requests

from counter_robots import _get_resource_path
from counter_robots.config import config


def update_lists():
    """Update the files containing the list of robots and machines."""
    update_file(config['ROBOTS_URL'], config['ROBOTS_FILE'])
    update_file(config['MACHINE_URL'], config['ROBOTS_FILE'])


def update_file(url, filename):
    """Update the content of a single file."""
    resp = requests.get(url)
    if resp.status_code != 200:
        raise Exception('GET {} failed.'.format(url))
    lines = resp.text.splitlines()
    lines = [s for s in lines if not s.startswith('#')]
    with open(_get_resource_path(filename), "w+") as f:
        for line in lines:
            f.write(line+'\n')



# -*- coding: utf-8 -*-
#
# This file is part of COUNTER-Robots.
# Copyright (C) 2018 CERN.
#
# COUNTER-Robots is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Library for COUNTER-compliant detection of machines and robots."""

from __future__ import absolute_import, print_function

from os.path import join
import pkg_resources
import re

from .version import __version__


config = dict(
    ROBOTS_URL='https://raw.githubusercontent.com/CDLUC3/Make-Data-Count/'
               'master/user-agents/lists/robot.txt',
    MACHINE_URL='https://raw.githubusercontent.com/CDLUC3/Make-Data-Count/'
                'master/user-agents/lists/machine.txt',
    ROBOTS_FILE='robot.txt',
    MACHINES_FILE='machine.txt'
)


def _get_resource_content(filename):
    """Retrieve content from a file in this Python package."""
    return pkg_resources.resource_string(__name__, join('data', filename))


def _get_resource_path(filename):
    """Retrieve path of a file in this Python package."""
    return pkg_resources.resource_filename(__name__, join('data', filename))


def _regexp(filename):
    """Get a list of patterns from a file and make a regular expression."""
    lines = _get_resource_content(filename).decode('utf-8').splitlines()
    return re.compile('|'.join(lines))


robots_regexp = _regexp(config['ROBOTS_FILE'])
machines_regexp = _regexp(config['MACHINES_FILE'])


def is_robot(user_agent):
    """Detect if the user agent belongs to a robot."""
    if user_agent is None:
        return False
    return bool(robots_regexp.search(user_agent))


def is_machine(user_agent):
    """Detect if a user agent belongs to a machine."""
    if user_agent is None:
        return True
    return bool(machines_regexp.search(user_agent))


__all__ = ('__version__', )



# -*- coding: utf-8 -*-
#
# This file is part of COUNTER-Robots.
# Copyright (C) 2018 CERN.
#
# COUNTER-Robots is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test counter robots."""

import pytest

from counter_robots import _get_resource_content, is_machine, is_robot


def test_version():
    """Test version string."""
    from counter_robots import __version__
    assert __version__


def test_get_resource_content():
    """Test version string."""
    data = _get_resource_content('machine.txt')
    assert data


def test_is_robot():
    machine_ua = 'Wget/1.14 (linux-gnu)'
    robot_ua = 'AdsBot-Google (+http://www.google.com/adsbot.html)'
    assert is_robot(machine_ua) is not True
    assert is_robot(robot_ua) is True


def test_is_machine():
    machine_ua = 'Wget/1.14 (linux-gnu)'
    robot_ua = 'AdsBot-Google (+http://www.google.com/adsbot.html)'
    assert is_machine(machine_ua) is True
    assert is_machine(robot_ua) is not True
